
/*
** Distributed with 'dig' version 2.0 from University of Southern
** California Information Sciences Institute (USC-ISI). 9/1/90
*/

/*
#define PARAMH	 "param.h"
*/
#define PARAMH   <sys/param.h>


#if 0
#define RESOLVH  <resolv.h>
#define NAMESERH <arpa/nameser.h>
#define NETDBH   <netdb.h>
#else
#define RESOLVH  "resolv.h"
#define NAMESERH "nameser.h"
#define NETDBH   "netdb.h"
#endif
